__version__ = '1.9.0'

import os
import re
import sys
import copy
import atexit
import signal
import getopt
import collections

from .isddc_dict import IsddcDict as idict
from .isddc_command import IsddcCommand as icmd

from . import isddc_baseclass as ibase
from . import isddc_time as itime
from . import isddc_os as ios
from . import isddc_log as ilog
from . import isddc_pool as ipool
from . import isddc_decorator as idec
from . import isddc_import as iimport
from . import isddc_crypto as icrypto
from . import isddc_obfuscer as iobfuscer

iapi: 'isddc api'
issh: 'isddc ssh'
ikube: 'isddc kube'
ikafka: 'isddc kafka'
imongo: 'isddc mongo'
imysql: 'isddc mysql'
iinflux: 'isddc influx'
idbpool: 'isddc dbpool'


def __init__(config: dict):
    config = copy.deepcopy(config)

    for iname, iconf in config.items():
        try:
            imodule = iimport.module(f'{__name__}.isddc_{iname[1:]}')
        except ModuleNotFoundError:
            continue

        init: dict = iconf.pop('init', {})

        for name, params in iconf.items():
            for item in init.items():
                params.setdefault(*item)

            iobj = imodule.__init__(params)

            if not hasattr(imodule, '__default__'):
                setattr(imodule, '__default__', iobj)

            setattr(imodule, name, iobj)

        setattr(sys.modules[__name__], iname, imodule)


def prt(*args, color: int = 31, font: int = 0,
        end: str = '\n', sep: str = ' ', file: open = None):
    """
    Custom print font and color.

    Color:
        31-red 32-green 34-blue 37-Gray 38-black
    Font:
        0-normal 1-bold
    sep:
        string inserted between values, default a space.
    """
    info: str = sep.join(str(i) for i in args)
    format: str = f'\033[{font};{color};0m{info}\033[0m'
    print(format, end=end, file=file)


class DictInterProcess(metaclass=ibase.FuncMode):
    """Dictionary internal processing
    Used to batch process values inside a dictionary.
    :param data: A dict.
    :param func: Your handler functions, ps:
        def func(name, value):
            if name == 'xxx':
                return process(value)
    """

    def __init__(self, data: dict, func=None):
        self.data = data
        self.func = func

    def __call__(self) -> dict:
        return self.one(self.data)

    def one(self, data: dict):
        for name, value in data.items():
            data[name] = self.two(name, value)

        return data

    def two(self, name=None, value={}):
        value = self.func(name, value) or value

        if isinstance(value, dict):
            return self.one(value)

        if isinstance(value, collections.Iterable) \
                and not isinstance(value, str):
            return [self.two(value=v) for v in value]

        return value


def in_container() -> bool:
    """Whether to run in the container."""
    if os.name == 'posix' and not icmd('fdisk -l').output:
        return True
    return False


def hump(name: str) -> str:
    if '_' in name:
        name: str = ''.join(_.capitalize() for _ in name.split('_'))

    if '-' in name:
        name: str = ''.join(_.capitalize() for _ in name.split('-'))

    return name[0].upper() + name[1:]


def underline(name: str) -> str:
    for _ in re.findall(r'([A-Z]+)[A-Z]', name):
        print(_)
        name = name.replace(_, _.capitalize())

    result: list = re.findall(r'[A-Z][a-z]+|[a-z]+|[A-Z]\d', name)

    return '_'.join(_.lower() for _ in result)


def get_caller(level: int = 1) -> str:
    back = sys._getframe().f_back

    for _ in range(level):
        back = back.f_back

    return back.f_code.co_name


def avg(numbers: list) -> float:
    return sum(numbers) / len(numbers)


class OverFunc(metaclass=ibase.FuncMode):
    # Functions in `__over_func__` are
    # executed at the end of the program.
    __over_func__ = []

    def __init__(self, func, *a, **kw):
        """Add a function that will execute
        after the main program is finished"""
        self.__over_func__.append((func, a, kw))

    def __call__(self):
        pass

    @staticmethod
    @atexit.register
    def __exec__():
        """Will execute when the program exits normally."""
        for func, a, kw in OverFunc.__over_func__:
            try:
                func(*a, **kw)
            except Exception as e:
                ilog.sine.error(f'{func.__name__} error: {e}')

    # Trigger `__exec__` function after being killed.
    signal.signal(signal.SIGTERM, lambda signum, stack_frame: exit(1))


def get_boot_params(*params) -> dict:
    boot_params: list = getopt.getopt(
        sys.argv[1:], '', (f'{x}=' for x in params))[0]
    return dict((n[2:], v) for n, v in boot_params)
